#!BPY

"""
Name: 'XVR Object File Format (.aam)...'
Blender: 232
Group: 'Export'
Tooltip: 'Export selected mesh to AAM file format (.aam) for the XVR engine'
"""

__author__ = "Walter Aprile (baff0)"
__url__ = ("blender", "elysiun",
"Author's homepage, http://www.supermoderno.com/")
__version__ = "0.1"

__bpydoc__ = """\
This script exports a mesh to the AAM file format, for use inside
the XVR graphic engine, under development by PERCRO at SSSUP.
http://www.percro.org/

Usage:<br>
    Select mesh to be exported and run this script from "File->Export" menu.

Notes:<br>
    Only exports a single selected mesh.
"""


# +---------------------------------------------------------+
# | Copyright (c) 2005 Walter Aprile                        |
# |                                                         |
# | walter.aprile@sssup.it                                  |
# |                                                         |
# | Released under the Blender Artistic Licence (BAL)       |
# |                                                         |
# +---------------------------------------------------------+
# | Read and write .AAM file format (*.aam)                 |
# +---------------------------------------------------------+

import Blender, mod_meshtools
import time

defaultMaterial = """MATERIALS
MatCount: 1
Mat# 0
{
    Name: DefaultExportedMaterial
    Class: Shell Material
    Am: 1.0000 1.0000 1.0000
    Di: 1.0000 1.0000 1.0000
    Sp: 0.0000 0.0000 0.0000
    Tr: 0.0000
    Sh: 0.1000
    Tx: N
    TS: N
}
ENDMATERIALS
"""


# ==============================
# ====== Write AAM Format ======
# ==============================
def write(filename):
    splittedCount = 0
    start = time.clock()
    file = open(filename, "wb")
    objects = Blender.Object.GetSelected()
    objname = objects[0].name
    meshname = objects[0].data.name
    mesh = Blender.NMesh.GetRaw(meshname)
    obj = Blender.Object.Get(objname)

    # === AAM Header ===
    # there is no header currently in the file format
    
    # == Mandatory materials section ==
    file.write(defaultMaterial)
    file.write("GEOMETRY\n")
    file.write("NObj: 1\n")
    file.write("NFrames: 1\n")
    file.write("Animation_mode: None\n")
    file.write("Frame: 0\n")
    file.write("{\n")
    file.write(" Obj: 0 " + objname + "\n")
    file.write(" Par: -1\n")
    file.write(" {\n")
    file.write("  MatID: 0\n") 


    # === Vertex List ===

    file.write("  V_List: " + str(len(mesh.verts)) + "\n")
    for i in range(len(mesh.verts)):
        if ((not i%100) and mod_meshtools.show_progress):
            Blender.Window.DrawProgressBar(float(i)/len(mesh.verts), "Writing Verts")
        x, y, z = mesh.verts[i].co
        file.write("   %f %f %f\n" % (x, y, z))

    # === No texture vertices for now ===

    file.write("   TV_List: 0\n")
    
    # === Face List ===

    # === Preprocess Face list for triangulization and cleanup

    def getIndex(v):
        return(v.index)
      
    faceList = []
    for face in mesh.faces:
        vertices = map(getIndex, face.v)
        if (len(vertices) == 4):
            splittedCount+=1
            ## Need to triangulize
            (verList1, verList2) = triangulize(vertices)
            if verticesNotDegenerate(verList1):
                faceList.append(verList1)
            if verticesNotDegenerate(verList2):
                faceList.append(verList2)
        else:
            if verticesNotDegenerate(vertices):
                faceList.append(vertices)
    
    # == Write out facelist

    faceCount = len(faceList)
    
    file.write("   I_List: " + str(faceCount) + " 1\n")
    file.write("   NEWGROUP: 0\n")

    i=0
    
    for face in faceList:
        i+=1
        if not i%100:
            Blender.Window.DrawProgressBar(float(i)/len(mesh.faces), "Writing Faces")
            
        file.write("  I: ")

        # vertices are in the normal order
        for vertex in face:
            file.write(str(vertex) +' ')
            group = 0 # Smoothing group, not used for now

        file.write(str(group))
        file.write("\n")

    file.write(" ENDGROUP\n")
    file.write("}\n")
    file.write("}\n")
    file.write("ENDGEOMETRY\n")
    Blender.Window.DrawProgressBar(1.0, '')  # clear progressbar
    file.close()
    end = time.clock()
    
    seconds = " in %.2f %s" % (end-start, "seconds")
    message = "Successfully exported " + Blender.sys.basename(filename) + seconds
    mod_meshtools.print_boxed("Splitted " + str(splittedCount) + " vertices.\n")
    mod_meshtools.print_boxed(message)

def triangulize(FourVertices):
    # Simplistic way to turn a quadrangle into two triangles
    return((FourVertices[0:3],FourVertices[2:4]+[FourVertices[0]]))

def verticesNotDegenerate(ThreeVertices):
    # check that triangle is not degenerate (could be much more refined)
    (a,b,c) = ThreeVertices
    if((a==b) or (b==c) or (a==c)):
        debug("Degenerate vertex omitted from vertex list")
        return (False)
    else:
        return(True)

def debug(aString):
    print ("! " + str(aString))

def fs_callback(filename):
    if (filename.find('.aam', -4) < 0):
        filename += '.aam'
    write(filename)

Blender.Window.FileSelector(fs_callback, "Export AAM")
